@@ -136,3 +136,50 @@ def find_connecting_edges(res_graph, molecule, nodes):
                 edges.append((high_res_node_a, high_res_node_b))
 
     return edges
+
+def _compute_path_length_cartesian(mol_idx, path, nonbond_matrix):
+    """
+    Computes the maximum length a graph path based on the super-CG model
+    step length. This is equivalent to the contour length of the super
+    CG model.
+    """
+    path_length = 0
+    for node_from, node_to in path:
+        path_length += nonbond_matrix.get_interaction(mol_idx,
+                                                      mol_idx,
+                                                      node_from,
+                                                      node_to)[0]
+    return path_length
+
+def compute_avg_step_length(molecule, mol_idx, nonbond_matrix, path):
+    """
+    Computes an average step length on a path.
+    """
+    # compute the length of that path in cartesian space
+    max_path_length = _compute_path_length_cartesian(mol_idx,
+                                                     path,
+                                                     nonbond_matrix)
+    # define range of end-to-end distances
+    # increment is the average step length
+    avg_step_length = max_path_length / len(path)
+    return avg_step_length, max_path_length
+
+def get_all_predecessors(graph, node, start_node=0):
+    """
+    Find all predecessor nodes of node in graph, given the
+    start_node.
+
+    Parameters
+    ----------
+    graph: nx.DiGraph
+    node: abc.hashable
+    start_node: abc.hashable
+    """
+    predecessors = [node]
+    while True:
+        pre_node = list(graph.predecessors(predecessors[-1]))[0]
+        predecessors.append(pre_node)
+        if pre_node == start_node:
+            break
+    predecessors.reverse()
+    return predecessors
