@@ -74,6 +74,16 @@ def _interpret_residue_mapping(graph, resname, new_residues):
                 atom_name_to_resname[node] = new_name
     return atom_name_to_resname
 
+def _find_starting_node(meta_molecule):
+    """
+    Find the first node that has coordinates if there is
+    otherwise return first node in list of nodes.
+    """
+    for node in meta_molecule.nodes:
+        if "build" not in meta_molecule.nodes[node]:
+            return node
+    return next(iter(meta_molecule.nodes()))
+
 class MetaMolecule(nx.Graph):
     """
     Graph that describes molecules at the residue level.
@@ -87,6 +97,9 @@ class MetaMolecule(nx.Graph):
         super().__init__(*args, **kwargs)
         self.molecule = None
         nx.set_node_attributes(self, True, "build")
+        self.__search_tree = None
+        self.root = None
+        self.dfs = False
 
     def add_monomer(self, current, resname, connections):
         """
@@ -172,6 +185,19 @@ class MetaMolecule(nx.Graph):
         self.relabel_and_redo_res_graph(mapping)
         return mapping
 
+    @property
+    def search_tree(self):
+
+        if self.__search_tree is None:
+            if self.root is None:
+                self.root =_find_starting_node(self)
+            if self.dfs:
+                self.__search_tree = nx.bfs_tree(self, source=self.root)
+            else:
+                self.__search_tree = nx.dfs_tree(self, source=self.root)
+
+        return self.__search_tree
+
     @staticmethod
     def _block_graph_to_res_graph(block):
         """
